#include <stdio.h>
#include <stdlib.h>	/* EXIT_FAILURE */

#include <string.h>	/* memset(3) */

#include <errno.h>

#include <assert.h>

#include <fcntl.h>	/* close(2) */

#include <sys/queue.h>
#include <sys/time.h>	/* struct timeval */
#include <sys/socket.h>	/* AF_INET socklen_t struct sockaddr
			 * struct sockaddr_storage */

#include <netinet/in.h>	/* struct sockaddr_in */

#include <unistd.h>	/* close(2) */

#include <err.h>	/* err(3) errx(3) */

#include <event.h>	/* struct event struct event_base event_init(3)
			 * event_base_loop(3) */

#include <arena/proto.h>

#include <socket.h>
#include <lookup.h>
#include <tls.h>


static int pending	= 2;


static void main_catch_accept(int fd, struct connect_info *i, enum socket_errno errnum, void *arg) {
	if (errnum == SOCKET_ECANCELLED)
		return; /* void */;

	if (errnum != SOCKET_ESUCCESS)
		err(EXIT_FAILURE, "socket_accept: %s", socket_errlist[errnum]);

	if (i->tls) {
		warnx("caught TLS accept");

		tls_close(i->tls);
 		i->tls	= 0;
	} else
		warnx("caught accept");

	(void)close(fd);

	if (--pending == 0)
		socket_close(arg);

	return /* void */;
} /* main_catch_accept() */


static void main_catch_connect(int fd, struct connect_info *i, enum socket_errno err, void *arg) {
	assert(err == SOCKET_ESUCCESS);

	if (i->tls) {
		warnx("caught TLS connect");

		tls_close(i->tls);
		i->tls	= 0;
	} else
		warnx("caught connect");

	(void)close(fd);

	if (--pending == 0)
		socket_close(arg);

	return /* void */;
} /* main_catch_connect() */


int main(int argc, char *argv[]) {
	struct event_base *ev_base;
	struct tls_context *tlc;
	enum tls_errno tls_errno;
	struct socket *s;
	struct socket_name soname;
	struct sockaddr_in sin;
	struct socket_options opts;

	if (!(ev_base = event_init()))
		err(EXIT_FAILURE, "event_init");

	assert(0 == lookup_init(&lookup_defaults, ev_base, 0));


	opts			= socket_defaults;
	opts.sa_reuseaddr	= 1;

	if (0 == (s = socket_open(&opts, ev_base, ARENA_STDLIB)))
		err(EXIT_FAILURE, "socket_open");

	if (argc > 2) {
		assert(0 == tls_init());

		if (!(tlc = tls_context_open(&tls_defaults, 0, &tls_errno)))
			errx(EXIT_FAILURE, "tls_context_open: %s", tls_errlist[tls_errno]);

		assert(argc > 2);

		assert(0 == tls_context_certify(tlc, argv[1], argv[2]));

		socket_tls_enable(s, tlc);
	}

	(void)memset(&sin, '\0', sizeof sin);

	sin.sin_family		= AF_INET;
	sin.sin_port		= htons(4000);
	sin.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);

	(void)soname_set_addr(&soname, (struct sockaddr *)&sin, sizeof sin);

	if (0 != socket_accept(s, &soname, &main_catch_accept, s, 0))
		errx(EXIT_FAILURE, "socket_accept: %s", socket_strerror(s));

	(void)soname_set_host(&soname, "localhost", "4000");

	socket_connect(s, &soname, &main_catch_connect, s, 0);

	event_base_loop(ev_base, 0);

	return 0;
} /* main() */

