/* ==========================================================================
 * libevnet/src/bufio.h - Network server library for libevent.
 * --------------------------------------------------------------------------
 * Copyright (c) 2006  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#ifndef EVNET_BUFIO_H
#define EVNET_BUFIO_H


/* Declare some things that the caller might not necessarily care about. */
struct event_base;	/* defined by libevent */
struct arena_prototype;	/* defined by libarena */

struct tls;


extern unsigned int bufio_recurse_max;


enum bufio_errno {
	BUFIO_ESUCCESS,		/* All's well. */
	BUFIO_ESYSTEM,		/* System error. See errno(2). */
	BUFIO_ETIMEDOUT,	/* Operation timed out. */
	BUFIO_EPENDING,		/* Conflicting operation already pending. */
	BUFIO_ECANCELLED,	/* Operation was explicitly cancelled. */
	BUFIO_ENOMORE,		/* No more data (aka end-of-file). */
	BUFIO_ENOSPACE,		/* Cannot restart read operation for lack of space. */
	BUFIO_ENODATA,		/* Cannot restart write operation for lack of data. */
}; /* enum bufio_errno */

extern const char *bufio_errlist[];

extern const int bufio_nerr;


enum bufio_mode {
	BUFIO_READ,
	BUFIO_WRITE,
	BUFIO_ABORT,
}; /* enum bufio_mode */


enum bufio_action {
	BUFIO_COMPLETE	= 0,
	BUFIO_RESTART	= 1,
}; /* enum bufio_action */


struct bufio;


extern const struct bufio_options {
	unsigned int recurse_max;

	enum bufio_mode (*sys_mode)(struct bufio *, int fd, enum bufio_mode, void *);

	ssize_t (*sys_read)(struct bufio *, int fd, void *, size_t, void *);
	ssize_t (*sys_write)(struct bufio *, int fd, const void *, size_t, void *);
} bufio_defaults;


struct bufio *bufio_open(int, const struct bufio_options *, struct event_base *, const struct arena_prototype *);

enum bufio_errno bufio_close(struct bufio *);

void bufio_tls_start(struct bufio *, struct tls *);

typedef enum bufio_action (*bufio_read_cb)(struct bufio *, void *, size_t, enum bufio_errno, void *);

enum bufio_errno bufio_read(struct bufio *, void *, size_t, bufio_read_cb, void *, struct timeval *);

typedef enum bufio_action (*bufio_write_cb)(struct bufio *, const void *, size_t, enum bufio_errno, void *);

enum bufio_errno bufio_write(struct bufio *, const void *, size_t, bufio_write_cb, void *, struct timeval *);

typedef enum bufio_action (*bufio_gets_cb)(struct bufio *, const char *, size_t, enum bufio_errno, void *);

enum bufio_errno bufio_gets(struct bufio *, bufio_gets_cb, void *, struct timeval *);

int bufio_sys_errno(struct bufio *);

enum bufio_errno bufio_errno(struct bufio *);

const char *bufio_strerror(struct bufio *);


#endif /* EVNET_BUFIO_H */
